# Generated by Django 4.1.3 on 2022-12-16 18:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='event',
            name='component',
            field=models.SlugField(blank=True, default=None, max_length=250, null=True),
        ),
        migrations.AddField(
            model_name='event',
            name='component_id',
            field=models.IntegerField(blank=True, default=None, null=True),
        ),
    ]
