# Generated by Django 4.1.4 on 2023-12-19 17:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0012_rule_match_by'),
    ]

    operations = [
        migrations.AddField(
            model_name='rulecheck',
            name='is_required',
            field=models.IntegerField(default=0),
        ),
    ]
