# Generated by Django 4.1.4 on 2023-12-25 20:46

from django.db import migrations, models
import django.db.models.deletion
import rest.fields
import rest.models.base


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0017_rename_requires_topt_member_requires_totp'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserPassKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('uuid', models.CharField(max_length=255, unique=True)),
                ('rp_id', models.CharField(db_index=True, max_length=255)),
                ('is_enabled', models.BooleanField(db_index=True, default=True)),
                ('name', models.CharField(max_length=255)),
                ('platform', models.CharField(default='', max_length=255)),
                ('last_used', models.DateTimeField(default=None, null=True)),
                ('token', models.CharField(max_length=255)),
                ('info', rest.fields.JSONField(default=None, null=True)),
                ('member', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='passkeys', to='account.member')),
            ],
            bases=(models.Model, rest.models.base.RestModel),
        ),
    ]
