# Generated by Django 4.1.4 on 2023-11-15 20:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0015_memberdevice_buid'),
    ]

    operations = [
        migrations.AddField(
            model_name='authsession',
            name='buid',
            field=models.CharField(blank=True, default=None, max_length=127, null=True),
        ),
    ]
