# Generated by Django 4.1.3 on 2022-12-16 20:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0002_event_component_event_component_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='rule',
            name='action',
            field=models.CharField(default=None, max_length=200, null=True),
        ),
    ]
